
;														
;														
; PROJEKT RS232-TWI_INTEFACE							
; 01/2006												
; Bernhard.Erfurt@gmx.de								
;														

; PROTOKOLL	PC <===> RS232 Interface					
; 	Zeichen 1 ====>* (Vorspann)							
;	Zeichen 2 ====>Befehl								
;	Zeichen 3 ====>Adresse SLAVE						
;	Zeichen 3 ====>RESERVE								
;	Zeichen 4 ====>Anzahl zu sendende Zeichen 	(low)	
;	Zeichen 5 ====>Anzahl zu sendende Zeichen 	(high)	
;	Zeichen 6 ====>Anzahl zu empfangene Zeichen (low)	
;	Zeichen 7 ====>Anzahl zu empfangene Zeichen (high)	
;	Zeichen 8 ====># (Nachspann)						
;	Zeichen 9...==># DATEN								
;	Zeichen 9...==># (Nachspann)						

; Befehle												
; BEFEHL=   0 ==> DUMMY an PC senden (zur berprfung der Verbindung)	ANTWORT vom C==> 0 	
; BEFEHL=   1 ==> MCUCSR auslesen (Grund des letzten Resets	(z.B. WDR)	ANTWORT vom C==> WERT	
; BEFEHL=   2 ==> MCUCSR CLEAR											ANTWORT vom C==> 0 	
; BEFEHL=   3 ==> Spannung auslesen										ANTWORT vom C==> WERT	
; BEFEHL=   4 ==> NAME des Interfaces auslesen							ANTWORT vom C==> WERT (16 Zeichen)
; BEFEHL=   5 ==> TWI - AUS												ANTWORT vom C==> 0		
; BEFEHL=   8 ==> TWI-BUS-PINS-ZUSTAND auslesen 						ANTWORT vom C==> WERT	(0=OK 1=SDA 2=SCL 3=SDA+SCL)
; BEFEHL=   9 ==> TWI-BUS-SPANNUNG-RESET (Versorgungsspannung fr SLAVE)ANTWORT vom C==> 0		
; BEFEHL=  10 ==> Reset C												ANTWORT vom C==> 0		
; BEFEHL=  11 ==> Reset BETRIEBSSPANNUNG 								ANTWORT vom C==> 0		
; BEFEHL=  12 ==> TAKT auslesen											ANTWORT vom C==> WERT (Byte1+2+3+4)
; BEFEHL=  15 ==> Fehlercode auslesen									ANTWORT vom C==> WERT	
;																								
; BEFEHL=  20 ==> TWBR lesen											ANTWORT vom C==> WERT	
; BEFEHL=  21 ==> TWSR lesen											ANTWORT vom C==> WERT	
; BEFEHL=  22 ==> TWAR lesen											ANTWORT vom C==> WERT	
; BEFEHL=  23 ==> TWCR lesen											ANTWORT vom C==> WERT	
; BEFEHL=  24 ==> TWDR lesen											ANTWORT vom C==> WERT	
;																								
; BEFEHL=  30 ==> TWBR setzen											ANTWORT vom C==> 0		
; BEFEHL=  31 ==> TWSR setzen											ANTWORT vom C==> 0		
; BEFEHL=  32 ==> TWAR setzen											ANTWORT vom C==> 0		
; BEFEHL=  33 ==> TWCR setzen											ANTWORT vom C==> 0		
; BEFEHL=  34 ==> TWDR setzen											ANTWORT vom C==> 0		
;																								
; BEFEHL=  50 ==> START MT + DATEN + STOP								ANTWORT vom C==> 0+0   
; BEFEHL=  60 ==> START MT + DATEN										ANTWORT vom C==> 0+0   
; BEFEHL=  65 ==> DATEN an SLAVE senden									ANTWORT vom C==> 0+0   
; BEFEHL=  80 ==> START MR + DATEN + STOP 								ANTWORT vom C==> 0+Byte1+Byte2+...+0   
; BEFEHL= 100 ==> STOP													ANTWORT vom C==> 0+0   


; BEISPIEL: 4 Zeichen (1+2+3+4) sind an den Slave 88 zu senden		
;  *(50)(88)(0)(4)(0)(0)(0)#(1)(2)(3)(4) #						 	

; FEHLERCODES											
; FEHLER=  0 ==> kein Fehler							
; FEHLER= 30 ==> falscher Befehl vom PC					


; PIN-BELEGUNG											
;														
; PB0	= 												
; PB1	= 												
; PB2	= 												
; PB3	= 												
; PB4	= 												
; PB5	= 												
; PB6	= 	Quarz										
; PB7	= 	Quarz										
;														
; PC0	= 												
; PC1	= 												
; PC3	= 												
; PC4	= TWI-SDA										
; PC5	= TWI_SCL										
;														
; PD0	= RXD+LED-GELB									
; PD1	= TXD+LED-GELB									
; PD2	=												
; PD3	=												
; PD4	=												
; PD5	= LED-GRN										
; PD6	= LED-GELB 										
; PD7	= LED-ROT										

.include "m8def.inc"

; C-Takt						
.equ TAKT=16000000

; TWI MASTER EINSTELLUNGEN 			  SCLfreq = CPUclk freq / (16 + 2(TWBR) * TWPS)	
.equ TWI_BIT_RATE			= 49	; Bit-Rate (sollte grer als 10 sein)			
.equ TWI_PRESCALER			= 2		; Vorteiler: 0=1 1=4 2=16 3=64					
; TWI SLAVE EINSTELLUNGEN		
.equ TWI_SLAVE_ADRESSE				= 0		; eigene SLAVE ADRESSE 1...127	(0=aus)	
.equ TWI_GENERAL_CALL_enable		= 0		; 1=Generral Call enabled / 0=disabled	
; USART							
.equ BAUD		=19200
; LEDs							
.equ LED_D_GRUEN	=5
.equ LED_D_GELB		=6
.equ LED_D_ROT		=7
; PINS							
.equ C_TWI_SDA		=4
.equ C_TWI_SCL		=5
.equ D_STEUERPIN_BETRIEBSSPANNUNG	=4  	;STEUERPIN fr SLAVE-Betriebsspannung (HIGH=> Betriebsspannung aus)
; SRAM - BELEGUNG				
.equ adr_DATEN					= 0x0060			; 0x0060 START Adresse 	
.equ adr_ADC_BYTE_1 			= 0x0060+0			; BYTE1 fr ADC MITTELWERTBILDUNG	
.equ adr_ADC_BYTE_2 	 		= 0x0060+1			; BYTE2 fr ADC MITTELWERTBILDUNG	
.equ adr_ADC_BYTE_3 			= 0x0060+2			; BYTE3 fr ADC MITTELWERTBILDUNG	
.equ adr_TWI_BEREICH			= 0x0060+3			; BEREICH fr TWI		


; REGISTER						
.def TAB_W	    = R0	; Tabellenwert	
.def FEHLER	    = R1	; Fehlercode	


.def SPANNUNG	= R6	; Betriebsspannung (90= 9,0 Volt)	

.def ZFG		= R7 	; ZeichenFolge gltig? 					
.def Z_ANZAHL	= R8	; Anzahl der Zeichen					

.def BEFEHL		= R9	; Zeichen 1 Befehl						
.def ADR_SLAVE	= R10	; Zeichen 2	Adresse Slave				
.def RESERVE	= R11	; Zeichen 3	RESERVE						
.def ANZ_S_L	= R12	; Zeichen 4								
.def ANZ_S_H	= R13	; Zeichen 5								
.def ANZ_E_L	= R14	; Zeichen 6								
.def ANZ_E_H	= R15	; Zeichen 6								

.def temp 		= R16 	; TEMP									
.def temp1		= R17 	; TEMP1									
.def temp2 		= R18 	; TEMP2									
.def temp3 		= R19 	; TEMP3									
.def temp4 		= R20 	; TEMP4									
.def temp5 		= R21 	; TEMP5									
.def temp6 		= R22 	; TEMP6									
.def temp7		= R23	; TEMP7 		 						
.def temp8		= R24	; TEMP8 		 						
.def temp9		= R25	; TEMP9 		 						

.cseg					;Beginn eines Code-Segmentes			
.org 0					;Startadresse=0							

rjmp RESET      ; Reset Handler                           
reti ;rjmp EXT_INT0   	  ; IRQ0 Handler                      
reti ;rjmp EXT_INT1   ; IRQ1 Handler                      
reti ;rjmp TIM2_COMP  ; Timer2 Compare Handler            
reti ;rjmp TIM2_OVF   ; Timer2 Overflow Handler           
reti ;rjmp TIM1_CAPT  ; Timer1 Capture Handler            
reti ;rjmp TIM1_COMPA ; Timer1 CompareA Handler           
reti ;rjmp TIM1_COMPB ; Timer1 CompareB Handler           
reti ;rjmp TIM1_OVF   ; Timer1 Overflow Handler           
reti ;rjmp TIM0_OVF   ; Timer0 Overflow Handler           
reti ;rjmp SPI_STC    ; SPI Transfer Complete Handler     
rjmp ZEICHEN_EINLESEN ; USART RX Complete Handler         
reti ;rjmp USART_UDRE ; UDR Empty Handler                 
reti ;rjmp USART_TXC  ; USART TX Complete Handler         
reti ;rjmp ADC        ; ADC Conversion Complete Handler   
reti ;rjmp EE_RDY     ; EEPROM Ready Handler              
reti ;rjmp ANA_COMP   ; Analog Comparator Handler         
reti ;rjmp TWSI       	  ; Two-wire Serial Interface Handler 
reti ;rjmp SPM_RDY    ; Store Program Memory Ready Handler

RESET:                        
;STACK initialisieren		
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
; PORT-B initialisieren		
	ldi temp, 0b00000000	; alles Eingang		
	out DDRB, temp			
; PORT-C initialisieren		
	ldi temp, 0b00000000	; Messeingang / TWI	
	out DDRC, temp	
; PORT-D initialisieren		
	ldi temp, 0b11110010	; RXD/TXD/BETRIEBSSPANNUNG/LEDs		
	out DDRD, temp
; PORT D setzen				
	ldi temp, 0b00000010
	out PORTD, temp
;USART initialisieren		
	ldi temp,TAKT/(BAUD*16)-1  		; Baudrate 
	out UBRRL,temp
   	ldi temp, (1<<URSEL)|(3<<UCSZ0)	; Frame-Format: 8 Bit
    out UCSRC, temp
    sbi UCSRB, RXCIE                ; Interrupt bei Empfang      
    sbi UCSRB, RXEN 				; RX (Empfang) aktivieren                  
	sbi UCSRB, TXEN  				; TX aktivieren                  
; ADC initialisieren mit "Free Run" und Vorteiler 128 
; Bit 7 -ADEN: ADC Enable                             
; Bit 6 -ADSC: ADC Start Conversion                   
; Bit 5 -ADFR: ADC Free Running Select                
; Bit 4 -ADIF: ADC Interrzpt Flag                     
; Bit 3 -ADIE: ADC Interrupt Enable                   
; Bit 2 -ADPS2: Vorteiler                             
; Bit 1 -ADPS2: Vorteiler                             
; Bit 0 -ADPS2: Vorteiler                             
ldi temp, ((1<<ADEN)|(1<<ADSC)|(1<<ADFR)) + 7
out ADCSR, temp
; ADC initialisieren  ###########################################
; REFS1 REFS0 Voltage Reference Selection
;  0     0    AREF, Internal Vref turned off
;  0     1    AVCC with external capacitor at AREF pin
;  1     1    Internal 2.56V Voltage Reference with external capacitor at AREF pin
ldi temp, 0<<REFS1 | 0<<REFS0
out ADMUX, temp


; TWI-INITIALISIERUNG			
rcall TWI_INI


;Watchdog aktivieren (max. Zeit)
	ldi TEMP,(1<<WDCE)|(1<<WDE)  
	out wdtcr,TEMP       
	ldi TEMP,(1<<WDE) | (1<<WDP2)|(1<<WDP1)|(1<<WDP0) 
	out wdtcr,temp

; LED TEST						
	rcall WAIT_100ms
	sbi portD, LED_D_ROT	; ROT	
	rcall WAIT_100ms
	sbi portD, LED_D_GELB	; GELB	
	rcall WAIT_100ms
	sbi portD, LED_D_GRUEN 	; GRN 	
	rcall WAIT_100ms
	cbi portD, LED_D_GRUEN 	; GRN 	

; STARTWERTE					
	clr FEHLER
	clr Z_ANZAHL


; interrupts freigeben			
	sei


;#######################################################################
;#######################################################################
;#######################################################################
LOOP:
	WDR						; Watch-Dog-RESET			
	rcall MESSEN_0			; BETRIEBSSPANNUNG messen	
	rcall ANZEIGE_LED_GELB	; LED-GELB TWI-aktiv		
	rcall ANZEIGE_LED_ROT	; LED-ROT  FEHLER			

	tst ZFG					; ZFG gltig?	
	breq LOOP				; nein => zurck

	mov temp, BEFEHL
	cpi temp, 50			; BEFEHL=50 ?	
	breq LOOP_50			; ja => SPRUNG	
	cpi temp, 60			; BEFEHL=60 ?	
	breq LOOP_60			; ja => SPRUNG	
	cpi temp, 65			; BEFEHL=65 ?	
	breq LOOP_65			; ja => SPRUNG	
	cpi temp, 80			; BEFEHL=80 ?	
	breq LOOP_80			; ja => SPRUNG	
	cpi temp, 100			; BEFEHL=100?	
	breq LOOP_100			; ja => SPRUNG	

	ldi temp, 30			; Fehlercode=30 (unbekannter Befehl)
	mov FEHLER, temp
	rjmp LOOP_FERTIG
; ------------
LOOP_50:
	rcall BEFEHL_50
rjmp LOOP_FERTIG
; ------------
LOOP_60:
	rcall BEFEHL_60
rjmp LOOP_FERTIG
; ------------
LOOP_65:
	rcall BEFEHL_65
rjmp LOOP_FERTIG
; ------------
LOOP_80:
	rcall BEFEHL_80
rjmp LOOP_FERTIG
; ------------
LOOP_100:
	rcall BEFEHL_100
rjmp LOOP_FERTIG
; ------------
LOOP_FERTIG:
	; FEHLERCODE an PC senden
	mov temp1,FEHLER			; Fehlercode	
	rcall SENDEN				; INPUT: temp1	
	clr ZFG
	clr Z_ANZAHL
rjmp LOOP
;###################################################################
;###################################################################
;###################################################################
; 																	
; BETRIEBSSPANNUNG messen											
; 																	
MESSEN_0:


	in temp, ADMUX
	andi temp, 0b11100000
	subi temp, 0			; KANAL 0	
	out ADMUX, temp


	rcall MESSEN			; OUT: TEMP1 / TEMP2	
	
	mov SPANNUNG,temp1

ret
;###################################################################
;###################################################################
;###################################################################
; OUT: 	TEMP1 (low)  	
;		TEMP2			
; 						
MESSEN:

; SRAM WERTE lschen			
	clr temp
	STS adr_ADC_BYTE_1, temp 	; (low)
	STS adr_ADC_BYTE_2, temp 
	STS adr_ADC_BYTE_3, temp 
	clr temp7
MESSEN_w:

; neuen ADC-Wert lesen	 Wert verwerfen, da meist vom vorherigem ADC-Kanal
	sbi ADCSRA, ADIF    ; logisch "1" lscht ADIF flag !
MESSEN_ww:                  
	sbis ADCSRA, ADIF	; warten bis ADIF flag gesetzt
	rjmp MESSEN_ww
; neuen ADC-Wert lesen				
	sbi ADCSRA, ADIF    ; logisch "1" lscht ADIF flag !
MESSEN_www:                
	sbis ADCSR, ADIF 	; warten bis ADIF flag gesetzt
	rjmp MESSEN_www
; ADC einlesen:						
	in  temp1, ADCL 	; immer zuerst low byte lesen
	in  temp2, ADCH 	; danach das mittlerweile gesperrte high byte
	clr temp3
; Wert aus SRAM holen		
	LDS temp4, adr_ADC_BYTE_1 	; (low)
	LDS temp5, adr_ADC_BYTE_2
	LDS temp6, adr_ADC_BYTE_3
; Wert addieren				
	add temp4, temp1			; (low)
	adc temp5, temp2
	adc temp6, temp3
; WERT in SRAM schreiben	
	STS adr_ADC_BYTE_1, temp4 	; (low)
	STS adr_ADC_BYTE_2, temp5 
	STS adr_ADC_BYTE_3, temp6 

	dec temp7					; temp7 = 0			
	brne MESSEN_w				; nein=>Schleife 	
; Werte aus SRAM holen	und durch 256 teilen		
	LDS temp1, adr_ADC_BYTE_2 	; (low)
	LDS temp2, adr_ADC_BYTE_3
ret
;#######################################################################
;#######################################################################
;#######################################################################
; STATUSANZEIGE (TWI-aktiv)												
ANZEIGE_LED_GELB:
	in temp, TWCR	; TWCR laden
	tst temp
	breq ANZEIGE_LED_GELB_OFF
	brne ANZEIGE_LED_GELB_ON

ANZEIGE_LED_GELB_ON:
	cbi portD, LED_D_GELB	; GELB an	
	ret

ANZEIGE_LED_GELB_OFF:
	sbi portD, LED_D_GELB	; GELB aus	
	ret
ret
;#######################################################################
;#######################################################################
;#######################################################################
; STATUSANZEIGE (TWI-aktiv)												
ANZEIGE_LED_ROT:
	sbi portD, LED_D_ROT	; ROT aus					
	
	mov temp, SPANNUNG		; SPANNUNG					
	cpi temp, 70			; Vergleich 70 (7,0 Volt)	
	brlo ANZEIGE_LED_ROT_ON	; kleiner	=> LED an		

	tst FEHLER				; FEHLER					
	brne ANZEIGE_LED_ROT_ON	; >0		=> LED an		
	ret

ANZEIGE_LED_ROT_ON:
	cbi portD, LED_D_ROT	; ROT an	
ret
;#######################################################################
;#######################################################################
;#######################################################################
; START MT + DATEN + STOP												
BEFEHL_50:
	rcall BEFEHL_60
	rcall TWI_STOP	
	clr ZFG
ret
;#######################################################################
;#######################################################################
;#######################################################################
; START MT + DATEN														
BEFEHL_60:
	tst ZFG
	brne BEFEHL_60_w
	ret
BEFEHL_60_w:
	rcall TWI_MASTER_TRANSMITTER_START	; INPUT: Adresse Slave	
	
	rcall BEFEHL_65
ret
;#######################################################################
;#######################################################################
;#######################################################################
; DATEN an SLAVE senden													
BEFEHL_65:
; FEHLERPRFUNG		
	tst FEHLER
	breq BEFEHL_65_ww
	ret
; INITIALISIERUNG	
BEFEHL_65_ww:
	clr XL
	clr XH
	ldi ZL, low (adr_TWI_BEREICH)		; ZEIGER initialisieren			
	ldi ZH, HIGH(adr_TWI_BEREICH)		
; FEHLERPRFUNG		
BEFEHL_65_DATEN:
	tst FEHLER
	breq BEFEHL_65_HIGH
	ret
; VERGLEICH HIGH	
BEFEHL_65_HIGH:	
	cp XH, ANZ_S_H
	breq BEFEHL_65_LOW
	brlo BEFEHL_65_senden
	brsh BEFEHL_65_FERTIG
; VERGLEICH LOW		
BEFEHL_65_LOW:
	cp XL, ANZ_S_L
	breq BEFEHL_65_FERTIG
	brsh BEFEHL_65_FERTIG
	brlo BEFEHL_65_senden
; DATEN an SLAVE senden
BEFEHL_65_senden:
	adiw XL,1	; x +1	
	LD temp1, Z+							; laden und ZEIGER erhhen
	rcall TWI_MASTER_TRANSMITTER_DATEN		; INPUT:	temp1 <= DATEN	
	rjmp BEFEHL_65_DATEN
BEFEHL_65_FERTIG:
	clr ZFG

ret
;#######################################################################
;#######################################################################
;#######################################################################
; START MR + DATEN + STOP 
BEFEHL_80:
	tst ZFG
	brne BEFEHL_80_w
	ret
BEFEHL_80_w:
; FEHLERPRFUNG	(Anzahl der Empfangszeichen muss mindestens 1 sein)
	tst ANZ_E_L
	brne BEFEHL_80_ww
	tst ANZ_E_H
	brne BEFEHL_80_ww
	ldi temp, 1
	mov ANZ_E_L, temp	; Zwangszuweisung 
BEFEHL_80_ww:
	rcall TWI_MASTER_RECEIVER_START
	clr XL
	clr XH

BEFEHL_80_DATEN:
	adiw XL, 1	

BEFEHL_80_DATEN_H:
	cp XH, ANZ_E_H					; ?			
	breq BEFEHL_80_DATEN_L			; gleich  => SPRUNG (low)	
	rjmp BEFEHL_80_DATEN_KLEINER	;         => SPRUNG kleiner	
BEFEHL_80_DATEN_L:
	cp XL, ANZ_E_L					; ? 		
	breq BEFEHL_80_DATEN_GLEICH		; gleich  => einlesen (letztes Zeichen)
	rjmp BEFEHL_80_DATEN_KLEINER	;         => einlesen				   

BEFEHL_80_DATEN_KLEINER:	
	rcall TWI_MASTER_RECEIVER_DATENEMPFANG
	rcall SENDEN				; INPUT: temp1	
	rjmp BEFEHL_80_DATEN

BEFEHL_80_DATEN_GLEICH:	
	rcall TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN
	rcall SENDEN				; INPUT: temp1	

BEFEHL_80_DATEN_STOP:
	rcall TWI_STOP	
	clr ZFG
ret
;#######################################################################
;#######################################################################
;#######################################################################
; STOP		
BEFEHL_100:
	rcall TWI_STOP	
	clr ZFG
ret
;#######################################################################
;#######################################################################
;#######################################################################
; INPUT: temp1															
senden:
	sbis UCSRA,UDRE     ; Warten bis UDR frei             	
	rjmp senden			; Sprung							
	out UDR, temp1		; senden							
	WDR 				; Watch-Dog-RESET					
ret
;#######################################################################
;#######################################################################
;#######################################################################
; Interruptroutine: wird ausgefhrt sobald ein Byte ber das UART empfangen wurde
ZEICHEN_EINLESEN:
		
		push temp           ; temp auf dem Stack sichern            
		in TEMP,SREG		; Einlesen des SREG 
		push TEMP	    	; Schreiben von  SREG  im Stack (KOPIE)
		push temp1
		push temp2
		push temp3
		push temp4
		push temp5
		push temp6
		push temp7
		push temp8
		push temp9

;-------
		WDR 				; Watch-Dog-RESET							
		in temp1, UDR
 		rcall ZEICHEN_AUFBEREITEN
;-------
		pop temp9
		pop temp8
		pop temp7
		pop temp6
		pop temp5
		pop temp4
		pop temp3
		pop temp2
		pop temp1
		pop	TEMP			; LESEN von SREG vom STACK (KOPIE)
		out	SREG,TEMP		; Wiederherstellen von SREG       
		pop temp      		; temp wiederherstellen                    
reti                              
;########################################################################
;########################################################################
;########################################################################
; input:	 temp1														
ZEICHEN_AUFBEREITEN:

; Register lschen
		clr ZFG						; Zeichenfolge gltig

		
		mov temp2, Z_ANZAHL			; Anzahl der Zeichen					
		
		cpi temp2, 0				; ANZAHL=0 ?   *	
		breq ZEICHEN_AUFBEREITEN_0
		cpi temp2, 1				; ANZAHL=1 ?
		breq ZEICHEN_AUFBEREITEN_1
		cpi temp2, 2				; ANZAHL=2 ?
		breq ZEICHEN_AUFBEREITEN_2
		cpi temp2, 3
		breq ZEICHEN_AUFBEREITEN_3	; ANZAHL=3 ?
		cpi temp2, 4
		breq ZEICHEN_AUFBEREITEN_4	; ANZAHL=4 ?
		cpi temp2, 5
		breq ZEICHEN_AUFBEREITEN_5	; ANZAHL=5 ?
		cpi temp2, 6
		breq ZEICHEN_AUFBEREITEN_6	; ANZAHL=6 ?
		cpi temp2, 7
		breq ZEICHEN_AUFBEREITEN_7	; ANZAHL=7 ? 
		cpi temp2, 8
		breq ZEICHEN_AUFBEREITEN_8	; ANZAHL=8 ?   #	
		cpi temp2, 9
		breq ZEICHEN_AUFBEREITEN_9	; ANZAHL=9     # oder DATEN	
		cpi temp2, 10
		breq ZEICHEN_AUFBEREITEN_10	; ANZAHL=10    # oder DATEN	

		ret


	
		
ZEICHEN_AUFBEREITEN_0:; *
; Fehlerprfung
		inc Z_ANZAHL				; +1	
		cpi temp1,'*'				; = '*' ?
		brne ZEICHEN_AUFBEREITEN_0_FEHLER	; nein
		ret 
ZEICHEN_AUFBEREITEN_0_FEHLER:
		clr Z_ANZAHL
		ret
ZEICHEN_AUFBEREITEN_1:
		inc Z_ANZAHL				; +1	
		mov BEFEHL,temp1
		ret
ZEICHEN_AUFBEREITEN_2:
		inc Z_ANZAHL				; +1	
		mov ADR_SLAVE,temp1
		ret
ZEICHEN_AUFBEREITEN_3:
		inc Z_ANZAHL				; +1	
		mov RESERVE,temp1
		ret
ZEICHEN_AUFBEREITEN_4:
		inc Z_ANZAHL				; +1	
		mov ANZ_S_L,temp1
		ret
ZEICHEN_AUFBEREITEN_5:
		inc Z_ANZAHL				; +1	
		mov ANZ_S_H,temp1
		ret
ZEICHEN_AUFBEREITEN_6:
		inc Z_ANZAHL				; +1	
		mov ANZ_E_L,temp1
		ret
ZEICHEN_AUFBEREITEN_7:
		inc Z_ANZAHL				; +1	
		mov ANZ_E_H,temp1
		ret
ZEICHEN_AUFBEREITEN_8:
		inc Z_ANZAHL				; +1	
; Fehlerprfung	"#"	
		cpi temp1, '#'
		breq ZEICHEN_AUFBEREITEN_8_w
		clr Z_ANZAHL
		ldi temp1, 10 				; Fehlercode	
		rcall SENDEN				; INPUT: temp1	
		ret
; Hier wird geprft, ob ein Systembefehl (z.B. RESET) abgearbeitet werden soll
ZEICHEN_AUFBEREITEN_8_w:		
		rcall DATEN_AUSWERTEN		; DATEN_AUSWERTEN	
ZEICHEN_AUFBEREITEN_9:
		tst Z_ANZAHL ; =0? 
		brne ZEICHEN_AUFBEREITEN_9_w
		ret
; HIER wird geprft, wieviel DATEN vom USART noch empfangen werden mssen
ZEICHEN_AUFBEREITEN_9_w:
		inc Z_ANZAHL
		clr XL
		clr XH
		ldi ZL, low (adr_TWI_BEREICH)		; ZEIGER initialisieren			
		ldi ZH, HIGH(adr_TWI_BEREICH)		
		ret
; DATEN einlesen
ZEICHEN_AUFBEREITEN_10:
		adiw XL,1

; VERGLEICH maximal 1001 BYTES fr den SRAM		
ZEICHEN_AUFBEREITEN_10_1001_HIGH:
		ldi temp3, low(1001)
		ldi temp4,high(1001)
		cp XH, temp4
		brlo ZEICHEN_AUFBEREITEN_10_high
		brsh ZEICHEN_AUFBEREITEN_10_1001_LOW
ZEICHEN_AUFBEREITEN_10_1001_LOW:
		cp XL, temp3
		brlo ZEICHEN_AUFBEREITEN_10_high
		breq ZEICHEN_AUFBEREITEN_10_high
		; Fehler
		clr ZFG
		clr Z_ANZAHL
		clr BEFEHL
		clr ADR_SLAVE
		clr RESERVE
		clr ANZ_S_L
		clr ANZ_S_H
		clr ANZ_E_L
		clr ANZ_E_H
		ldi temp1,15	; FEHLERCODE			
		rcall SENDEN	; SENDEN INPUT: temp1	
		ret
; Vergleich HIGH 
ZEICHEN_AUFBEREITEN_10_high:
		cp XH, ANZ_S_H									; ?									
		breq ZEICHEN_AUFBEREITEN_10_low					; gleich => nchste Prfung (low)	
		brlo ZEICHEN_AUFBEREITEN_10_SRAM_schreiben		; kleiner=> speichern				
		; Fehler
		clr ZFG
		clr Z_ANZAHL
		clr BEFEHL
		clr ADR_SLAVE
		clr RESERVE
		clr ANZ_S_L
		clr ANZ_S_H
		clr ANZ_E_L
		clr ANZ_E_H
		ret 										
; Vergleich LOW  
ZEICHEN_AUFBEREITEN_10_low:
		cp XL, ANZ_S_L									; ?									
		brlo ZEICHEN_AUFBEREITEN_10_SRAM_schreiben		; kleiner=> speichern				
		breq ZEICHEN_AUFBEREITEN_10_SRAM_schreiben		; gleich => speichern				
		rjmp ZEICHEN_AUFBEREITEN_10_FEHLERPRUEFUNG		; grer => FEHLERPRFUNG			
		
; DATEN vom USART speichern im SRAM			
ZEICHEN_AUFBEREITEN_10_SRAM_schreiben:
		ST Z+, temp1	; SPEICHERN und Zeiger um 1 erhhen	


ret
; FEHELRPRFUNG 	"#"						
ZEICHEN_AUFBEREITEN_10_FEHLERPRUEFUNG:
		clr Z_ANZAHL
		cpi temp1,'#'				; = '#' ?
		breq ZEICHEN_AUFBEREITEN_10_kein_FEHLER 		
		brne ZEICHEN_AUFBEREITEN_10_FEHLER
; kein FEHLER								
ZEICHEN_AUFBEREITEN_10_kein_FEHLER:
		clr temp1		; FEHLERCODE			
		rcall SENDEN	; SENDEN INPUT: temp1	
		ldi temp,1
		mov ZFG,temp
		ret
ZEICHEN_AUFBEREITEN_10_FEHLER:
		ldi temp1,12	; FEHLERCODE			
		rcall SENDEN	; SENDEN INPUT: temp1	
		ret
;#######################################################################
;#######################################################################
;#######################################################################
; Input: 	Z1 Befehl													
;			Z2 ADRESSE-SLAVE											
;			R  Reserve													
;			Z3 ANZAHL-Sendebytes (low)									
;			Z4 ANZAHL-Sendebytes (high)									
;			Z5 ANZAHL-Empfangsbytes (low)								
;			Z6 ANZAHL-Empfangsbytes (HIGH								
;			Z7 DATEN2													
;			Z8 DATEN2													
DATEN_AUSWERTEN:
			
			
			
			mov temp, BEFEHL	

DATEN_AUSWERTEN_A_000:			
			cpi temp, 0
			brne DATEN_AUSWERTEN_A_001
			rjmp DATEN_AUSWERTEN_000		; =0  ? 
DATEN_AUSWERTEN_A_001:			
			cpi temp, 1
			brne DATEN_AUSWERTEN_A_002
			rjmp DATEN_AUSWERTEN_001		; =1  ? 
DATEN_AUSWERTEN_A_002:			
			cpi temp, 2
			brne DATEN_AUSWERTEN_A_003
			rjmp DATEN_AUSWERTEN_002		; =2  ? 
DATEN_AUSWERTEN_A_003:			
			cpi temp, 3
			brne DATEN_AUSWERTEN_A_004
			rjmp DATEN_AUSWERTEN_003		; =3  ? 
DATEN_AUSWERTEN_A_004:			
			cpi temp, 4
			brne DATEN_AUSWERTEN_A_005
			rjmp DATEN_AUSWERTEN_004		; =4  ? 
DATEN_AUSWERTEN_A_005:	
			cpi temp, 5
			brne DATEN_AUSWERTEN_A_008
			rjmp DATEN_AUSWERTEN_005		; =5  ? 
DATEN_AUSWERTEN_A_008:	
			cpi temp, 8
			brne DATEN_AUSWERTEN_A_009
			rjmp DATEN_AUSWERTEN_008		; =8  ? 
DATEN_AUSWERTEN_A_009:	
			cpi temp, 9
			brne DATEN_AUSWERTEN_A_010
			rjmp DATEN_AUSWERTEN_009		; =9  ? 
DATEN_AUSWERTEN_A_010:	
			cpi temp, 10
			brne DATEN_AUSWERTEN_A_012
			rjmp DATEN_AUSWERTEN_010		; =10 ? 
DATEN_AUSWERTEN_A_012:	
			cpi temp, 12
			brne DATEN_AUSWERTEN_A_015
			rjmp DATEN_AUSWERTEN_012		; =12 ? 
DATEN_AUSWERTEN_A_015:	
			cpi temp, 15
			brne DATEN_AUSWERTEN_A_020
			rjmp DATEN_AUSWERTEN_015		; =15 ? 
DATEN_AUSWERTEN_A_020:	
			cpi temp, 20
			brne DATEN_AUSWERTEN_A_021
			rjmp DATEN_AUSWERTEN_020		; =20 ? 
DATEN_AUSWERTEN_A_021:	
			cpi temp, 21
			brne DATEN_AUSWERTEN_A_022
			rjmp DATEN_AUSWERTEN_021		; =21 ? 
DATEN_AUSWERTEN_A_022:	
			cpi temp, 22
			brne DATEN_AUSWERTEN_A_023
			rjmp DATEN_AUSWERTEN_022		; =22 ? 
DATEN_AUSWERTEN_A_023:	
    		cpi temp, 23
			brne DATEN_AUSWERTEN_A_024
			rjmp DATEN_AUSWERTEN_023		; =23 ? 
DATEN_AUSWERTEN_A_024:	
			cpi temp, 24
			brne DATEN_AUSWERTEN_A_030
			rjmp DATEN_AUSWERTEN_024		; =24 ? 
DATEN_AUSWERTEN_A_030:	
			cpi temp, 30
			brne DATEN_AUSWERTEN_A_031
			rjmp DATEN_AUSWERTEN_030		; =30 ? 
DATEN_AUSWERTEN_A_031:	
			cpi temp, 31
			brne DATEN_AUSWERTEN_A_032
			rjmp DATEN_AUSWERTEN_031		; =31 ? 
DATEN_AUSWERTEN_A_032:	
			cpi temp, 32
			brne DATEN_AUSWERTEN_A_033
			rjmp DATEN_AUSWERTEN_032		; =32 ? 
DATEN_AUSWERTEN_A_033:	
			cpi temp, 33
			brne DATEN_AUSWERTEN_A_034
			rjmp DATEN_AUSWERTEN_033		; =33 ? 
DATEN_AUSWERTEN_A_034:	
			cpi temp, 34
			brne DATEN_AUSWERTEN_A_FERTIG
			rjmp DATEN_AUSWERTEN_034		; =34 ? 
DATEN_AUSWERTEN_A_FERTIG:	
			ret

DATEN_AUSWERTEN_000:;	 "0" => DUMMY an PC senden (zur berprfung der Verbindung)
	clr temp1
	rcall SENDEN				; SENDEN INPUT: temp1	
	rcall DATEN_AUSWERTEN_FERTIG
	ret
DATEN_AUSWERTEN_001:;	 "1" => MCUCSR auslesen (Grund des letzten Reset) z.B. Watchdog	oder RESET-TASTE
	in temp1, MCUCSR
	rcall SENDEN				; SENDEN INPUT: temp1	
	rcall DATEN_AUSWERTEN_FERTIG
	ret
DATEN_AUSWERTEN_002:;	 "2" => MCUCSR Clear	
	clr temp1
	out MCUCSR,temp1
	rcall SENDEN				; SENDEN INPUT: temp1	
	rcall DATEN_AUSWERTEN_FERTIG
	ret
DATEN_AUSWERTEN_003:;	 "3" => Betriebspannung auslesen	
	mov temp1, SPANNUNG
	rcall SENDEN				; SENDEN INPUT: temp1	
	rcall DATEN_AUSWERTEN_FERTIG
	ret
DATEN_AUSWERTEN_004:;	 "4" => NAME des INTERFACES auslesen (16 Zeichen)	
	push ZL									; sichern			
	push ZH									; sichern			
	ldi ZH, high(TABELLE_TWI_16_INFO*2)		; Z-Zeiger initialisieren	
	ldi ZL,  low(TABELLE_TWI_16_INFO*2)
DATEN_AUSWERTEN_004_w:
	lpm     								; Z=> nach R0 (gelesenes Byte ist danach in R0)	
	tst TAB_W  								; 0 ist Zeichen fr Ende der Zeichenkette		
	breq DATEN_AUSWERTEN_004_ww				; =0 ==> Sprung	
	mov temp1,TAB_w
	rcall SENDEN							; SENDEN INPUT: temp1	
	adiw ZL,1								; ZL erhhen
	rjmp DATEN_AUSWERTEN_004_w
DATEN_AUSWERTEN_004_ww:
	pop ZH									; zurckschreiben	
	pop ZL									; zurckschreiben	
	rcall DATEN_AUSWERTEN_FERTIG
	ret
DATEN_AUSWERTEN_005:;	 "5 => TWI - AUS
	clr temp1
	out TWCR,temp1
	clr temp1
	rcall SENDEN	; SENDEN INPUT: temp1				
	rcall DATEN_AUSWERTEN_FERTIG
	ret
DATEN_AUSWERTEN_008:;	 "8" => TWI-PINS_ABFRAGEN RUHEZUSTAND (SDA+SCL =HIGH)
	clr temp1
	sbis PINC,C_TWI_SDA	; SPRUNG, wenn SDA auf HIGH
	ldi temp1,1		; =1	
	sbis PINC,C_TWI_SCL	; SPRUNG, wenn SCL auf HIGH
	subi temp1,-2	; +2	
	rcall SENDEN	; SENDEN INPUT: temp1				
	rcall DATEN_AUSWERTEN_FERTIG
	ret


DATEN_AUSWERTEN_009:;	 "9" => TWI-BUS-Versorgungsspannung- RESET (SLAVES) abschalten 
	sbi PORTD,D_STEUERPIN_BETRIEBSSPANNUNG  ; PIN auf HIGH	
	rcall wait_1s							; warten
	cbi PORTD,D_STEUERPIN_BETRIEBSSPANNUNG  ; PIN auf LOW			
	clr temp1
	rcall SENDEN	; SENDEN INPUT: temp1				
	rcall DATEN_AUSWERTEN_FERTIG
	ret
DATEN_AUSWERTEN_010:; "10 => Reset C" 
	clr temp1
	rcall SENDEN	; SENDEN INPUT: temp1	
	clr temp
	out TWCR, temp	; TWI aus				
	rcall wait_100ms
	rcall DATEN_AUSWERTEN_FERTIG
	rjmp 0000


DATEN_AUSWERTEN_012:; "12 => TAKT auslesen
	ldi temp1, BYTE1(TAKT)
	rcall SENDEN	; SENDEN INPUT: temp1		
	ldi temp1, BYTE2(TAKT)
	rcall SENDEN	; SENDEN INPUT: temp1		
	ldi temp1, BYTE3(TAKT)
	rcall SENDEN	; SENDEN INPUT: temp1		
	ldi temp1, BYTE4(TAKT)
	rcall SENDEN	; SENDEN INPUT: temp1		
	rcall DATEN_AUSWERTEN_FERTIG
	ret

DATEN_AUSWERTEN_015:; "15 => Fehlercode auslesen
	mov temp1, FEHLER
	rcall SENDEN	; SENDEN INPUT: temp1		
	rcall DATEN_AUSWERTEN_FERTIG
	ret

DATEN_AUSWERTEN_020:; "20 => TWBR lesen" 
	in temp1, TWBR
	rcall SENDEN	; SENDEN INPUT: temp1			
	rcall DATEN_AUSWERTEN_FERTIG
	ret

DATEN_AUSWERTEN_021:; "21 => TWSR lesen" 
	in temp1, TWSR
	rcall SENDEN	; SENDEN INPUT: temp1			
	rcall DATEN_AUSWERTEN_FERTIG
	ret

DATEN_AUSWERTEN_022:; "22 => TWAR lesen" 
	in temp1, TWAR
	rcall SENDEN	; SENDEN INPUT: temp1				
	rcall DATEN_AUSWERTEN_FERTIG
	ret

DATEN_AUSWERTEN_023:; "23 => TWCR lesen" 
	in temp1, TWCR
	rcall SENDEN	; SENDEN INPUT: temp1				
	rcall DATEN_AUSWERTEN_FERTIG
	ret

DATEN_AUSWERTEN_024:; "24 => TWDR lesen" 
	in temp1, TWDR
	rcall SENDEN	; SENDEN INPUT: temp1				
	rcall DATEN_AUSWERTEN_FERTIG
	ret


DATEN_AUSWERTEN_030:; "30 => TWBR setzen" 
	out TWBR,RESERVE
	clr temp1
	rcall SENDEN	; SENDEN INPUT: temp1				
	rcall DATEN_AUSWERTEN_FERTIG
	ret

DATEN_AUSWERTEN_031:; "31 => TWSR setzen" 
	out TWSR,RESERVE
	clr temp1
	rcall SENDEN	; SENDEN INPUT: temp1				
	rcall DATEN_AUSWERTEN_FERTIG
	ret	
DATEN_AUSWERTEN_032:; "32 => TWAR setzen" 
	out TWAR,RESERVE
	clr temp1
	rcall SENDEN	; SENDEN INPUT: temp1				
	rcall DATEN_AUSWERTEN_FERTIG
	ret	
DATEN_AUSWERTEN_033:; "33 => TWCR setzen" 
	out TWCR,RESERVE
	clr temp1
	rcall SENDEN	; SENDEN INPUT: temp1				
	rcall DATEN_AUSWERTEN_FERTIG
	ret
DATEN_AUSWERTEN_034:; "34 => TWDR setzen" 
	out TWDR,RESERVE
	clr temp1
	rcall SENDEN	; SENDEN INPUT: temp1				
	rcall DATEN_AUSWERTEN_FERTIG
	ret

DATEN_AUSWERTEN_FERTIG:
	clr ZFG

	clr Z_ANZAHL
	clr BEFEHL
	clr ADR_SLAVE
	clr RESERVE
	clr ANZ_S_L
	clr ANZ_S_H
	clr ANZ_E_L
	clr ANZ_E_H
	ret
; #############################################################################
; #############################################################################
; #############################################################################
;																			   
; TWI/I2C START-ROUTINE														   
;																			   
; INPUT:	ADRESSE SLAVE											   			
;																			   
TWI_MASTER_TRANSMITTER_START:
; 	FEHLER lschen					
	clr FEHLER
; START								
	ldi temp, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)
	out TWCR, temp	
; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
; Check TWSR						
	in temp,TWSR													
	andi temp, 0xF8
	cpi temp,  0x08									; 0x08 START
	breq TWI_MASTER_TRANSMITTER_START_w
	cpi temp,  0x10									; 0x10 (repeated START)
	breq TWI_MASTER_TRANSMITTER_START_w
	brne TWI_MASTER_TRANSMITTER_START_ERROR_SCHWER	; ERROR SCHWER

TWI_MASTER_TRANSMITTER_START_w:
; Load SLA + "W"  (W=0) 			
	mov temp,ADR_SLAVE						; SLAVE ADRESSE 	
	lsl  temp								; 1 x links
	out TWDR, temp
; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	out TWCR, temp
; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
; Check TWSR						
	in temp,TWSR												
	andi temp, 0xF8
	cpi temp,  0x18							; 0x18
	brne TWI_MASTER_TRANSMITTER_START_ERROR	; ERROR
ret
; SCHWERER FEHLER
TWI_MASTER_TRANSMITTER_START_ERROR_SCHWER:
	ldi temp,100							; FEHLERCODE	
	mov FEHLER, temp
	rcall TWI_ERROR							; ERROR
ret
; FEHLER
TWI_MASTER_TRANSMITTER_START_ERROR:
	ldi temp,110							; FEHLERCODE	
	mov FEHLER, temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; Diese Routine wird verwendet,												   
; um ein Zeichen zum SLAVE zu senden										   
; 																			   
; 																			   
; INPUT:	temp1 <= DATEN													   
; 																			   
TWI_MASTER_TRANSMITTER_DATEN:
; FEHLERPRFUNG
	tst FEHLER								; Fehler =0 ?	
	breq TWI_MASTER_TRANSMITTER_DATEN_w		; ja			
	ret										; zurck		
TWI_MASTER_TRANSMITTER_DATEN_w:
; 	Load DATA						
	out TWDR, temp1							
; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	out TWCR, temp
; warten, bis TWINT wieder = 1		
	rcall TWI_TWINT_wait									
; Check TWSR						
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x28	; 0x28
	brne TWI_MASTER_TRANSMITTER_DATEN_ERROR
ret
TWI_MASTER_TRANSMITTER_DATEN_ERROR:
	ldi temp,120								; FEHLERCODE	
	mov FEHLER, temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																			   
; TWI/I2C START-ROUTINE														   
;																			   
; INPUT:	 ADRESSE SLAVE											   		   
;																			   
TWI_MASTER_RECEIVER_START:
; FEHLER lschen					
	clr FEHLER
; START 											
	ldi temp, (1<<TWINT | 1<<TWEA | 1<<TWSTA | 1<<TWEN )
	out TWCR, temp	
; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
; Check TWSR 										
	in temp,TWSR		
	andi temp, 0xF8
	cpi temp,  0x08 								; 0x08 START erfolgreich
	breq TWI_MASTER_RECEIVER_START_w
	cpi temp,  0x10									; 0x10 (repeated START)
	breq TWI_MASTER_RECEIVER_START_w
	rjmp TWI_MASTER_RECEIVER_START_ERROR_SCHWER		; ERROR schwer


TWI_MASTER_RECEIVER_START_w:

; Load SLA +"R" (R=1)								
	mov temp,ADR_SLAVE			; SLAVE ADRESSE 	
	lsl  temp			; 1 x links		
	subi temp, -1 		; "R" setzen	
	out TWDR, temp
; senden - Clear TWINT 								
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
; warten, bis TWINT wieder = 1						
	rcall TWI_TWINT_wait
; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x40 	;SLA+R gesendet; ACK empfangen
	brne TWI_MASTER_RECEIVER_START_ERROR	; ERROR
	ret
; schwerer FEHLER
TWI_MASTER_RECEIVER_START_ERROR_SCHWER:
	ldi temp,200							; FEHLERCODE	
	mov FEHLER, temp
	rcall TWI_ERROR							; ERROR
	ret
; FEHLER			
TWI_MASTER_RECEIVER_START_ERROR:
	ldi temp,210	
	mov FEHLER, temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; nur ein! Zeichen empfangen werden soll, oder wenn								
; das letzte! Zeichen empfangen werden soll										
;																				
; OUTPUT: temp1																	
;																				
TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN:
; VARIABLEN zurcksetzen										
	clr TEMP1
; Fehlerprfung													
	tst FEHLER
	breq TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_w
	ret
TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_w:
; senden "NOT ACK" (TWEA=0) fr letztes bzw fr ein Daten-Byte
	ldi temp, (1<<TWINT | 0<<TWEA  | 1<<TWEN )
	out TWCR, temp
; TWINT abfragen									
	rcall TWI_TWINT_wait
; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi  temp, 0x58	; letztes Data Byte empfangen-NOT ACK zurckgesendet
	brne TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_ERROR	; ERROR
; DATEN einlesen									
	in temp1, TWDR
	ret
; --------------------------------------------------
TWI_MASTER_RECEIVER_DATENEMPFANG_LETZTES_ZEICHEN_ERROR:
	ldi temp,220	
	mov FEHLER, temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; mehrere! Zeichen empfangen werden sollen										
;																				
; OUTPUT: temp1																	
;																				
TWI_MASTER_RECEIVER_DATENEMPFANG:
; VARIABLEN zurcksetzen							
	clr temp1
; Fehlerprfung										
	tst FEHLER
	breq TWI_MASTER_RECEIVER_DATENEMPFANG_w
	ret
TWI_MASTER_RECEIVER_DATENEMPFANG_w:
; senden "ACK" (TWEA=1) fr mehrere Daten-Bytes 	
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
; TWINT abfragen									
	rcall TWI_TWINT_wait
; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x50	; Data Byte empfangen- ACK zurckgesendet
	brne TWI_MASTER_RECEIVER_DATENEMPFANG_ERROR	; ERROR
; DATEN einlesen									
	in temp1, TWDR
	ret
;---------------------------------------------------
TWI_MASTER_RECEIVER_DATENEMPFANG_ERROR:
	ldi temp,210	
	mov FEHLER, temp
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; ERROR- Routine															   
; 																			   
TWI_ERROR:
	rcall TWI_STOP			; TWI-STOP	
	
	rcall wait_100ms

	ldi temp, (1<<TWINT)	; TWI-aus	
	out TWCR, temp	
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; STOP- Routine																   
; 																			   
TWI_STOP:
	ldi temp, (1<<TWINT)|(1<<TWSTO)|(1<<TWEN)	; STOP								
	out TWCR, temp	
ret
; #############################################################################
; #############################################################################
; #############################################################################
; 																			   
; warten, bis das TWINT Flag gesetzt ist									   
; 																			   
TWI_TWINT_wait:
	WDR 				; WATCH-DOG-RESET
	in temp, TWCR
	sbrs temp, TWINT
	rjmp TWI_TWINT_wait
ret
; #############################################################################
; #############################################################################
; #############################################################################
TWI_INI:
	; TWI TWBR 				Bit RATE Register	
	ldi temp, TWI_BIT_RATE
	out TWBR, temp
	; TWI TWSR 				STATUS-Register		
	ldi temp, TWI_PRESCALER
	out TWSR, temp
	; TWI TWAR (nur SLAVE) 	Adress-Register		
	ldi temp,(TWI_SLAVE_ADRESSE<<TWA0 | TWI_GENERAL_CALL_ENABLE<<TWGCE)
	out TWAR,temp	
	; TWI TWCR 				Control-REGISTER (fr MASTER alles aus)
	ldi temp, 1<<TWINT|0<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|0<<TWEN|0<<TWIE
	out TWCR,temp
ret
; #############################################################################
; #############################################################################
;																			   
; ca. 1s Pause																   
;																			   
WAIT_1s:
	push temp1           ; temp1 auf dem Stack sichern            
	push temp2           ; temp2 auf dem Stack sichern            
	push temp3           ; temp3 auf dem Stack sichern            

;------------
	ldi	temp3,80			
	clr temp2
	clr	temp1			
wait1s_w:	
	wdr					; WATCH-DOG-RESET
	dec	temp1
	brne	wait1s_w
	dec	temp2		
	brne	wait1s_w
	dec	temp3
	brne	wait1s_w
;------------
	pop temp3      		; temp3 wiederherstellen                    
	pop temp2      		; temp2 wiederherstellen                    
	pop temp1      		; temp1 wiederherstellen                    
ret
; #############################################################################
; #############################################################################
;																			   
; ca. 100ms Pause																   
;																			   
WAIT_100ms:
	push temp1           ; temp1 auf dem Stack sichern            
	push temp2           ; temp2 auf dem Stack sichern            
	push temp3           ; temp3 auf dem Stack sichern            

;------------
	ldi	temp3,6			
	clr temp2
	clr	temp1			
WAIT_100ms_w:	
	wdr					; WATCH-DOG-RESET
	dec	temp1
	brne	WAIT_100ms_w
	dec	temp2		
	brne	WAIT_100ms_w
	dec	temp3
	brne	WAIT_100ms_w
;------------
	pop temp3      		; temp3 wiederherstellen                    
	pop temp2      		; temp2 wiederherstellen                    
	pop temp1      		; temp1 wiederherstellen                    
ret


TABELLE_TWI_16_INFO:
.db "INTERFACE -00001",0
